@@ -295,7 +295,7 @@ static void bookmark_edit_item(struct dialog_data *dlg, struct list *data, void
 			break;
 
 		default:
-			internal("Unsupported dialog title.\n");
+			internal_error("Unsupported dialog title.\n");
 	}
 	d->fn = bookmark_edit_item_fn;
 	d->udata = item;  /* item */
@@ -782,7 +782,7 @@ static void save_bookmarks(struct session *ses)
 	} else {
 		if (ses) {
 			unsigned char *f = stracpy(bookmarks_file);
-			msg_box(ses->term, getml(f, NULL), TEXT_(T_BOOKMARK_ERROR), AL_CENTER, TEXT_(T_UNABLE_TO_WRITE_TO_BOOKMARK_FILE), cast_uchar " ", f, cast_uchar ": ", get_err_msg(err), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
+			msg_box(ses->term, getml(f, NULL), TEXT_(T_BOOKMARK_ERROR), AL_CENTER, TEXT_(T_UNABLE_TO_WRITE_TO_BOOKMARK_FILE), cast_uchar " ", f, cast_uchar ": ", get_err_msg(err, ses->term), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		}
 	}
 
