@@ -63,8 +63,6 @@ int set_cookie(struct terminal *term, unsigned char *url, unsigned char *str)
 	date = parse_header_param(str, cast_uchar "expires", 0);
 	if (date) {
 		cookie->expires = parse_http_date(date);
-		/* kdo tohle napsal a proc ?? */
-		/*if (! cookie->expires) cookie->expires++;*/ /* no harm and we can use zero then */
 		mem_free(date);
 	} else {
 		cookie->expires = 0;
@@ -160,12 +158,12 @@ int is_path_prefix(unsigned char *d, unsigned char *s)
 	return d[dl - 1] == '/' || !s[dl] || s[dl] == '/' || s[dl] == POST_CHAR || s[dl] == '?' || s[dl] == '&';
 }
 
-int cookie_expired(struct cookie *c)	/* parse_http_date is broken */
+int cookie_expired(struct cookie *c)
 {
 	time_t t;
 	errno = 0;
 	EINTRLOOPX(t, time(NULL), (time_t)-1);
-	return 0 && (c->expires && c->expires < t);
+	return c->expires && c->expires < t;
 }
 
 void add_cookies(unsigned char **s, int *l, unsigned char *url)
@@ -194,7 +192,7 @@ void add_cookies(unsigned char **s, int *l, unsigned char *url)
 		else add_to_str(s, l, cast_uchar "; ");
 		add_to_str(s, l, c->name);
 		if (c->value) {
-			add_to_str(s, l, cast_uchar "=");
+			add_chr_to_str(s, l, '=');
 			add_to_str(s, l, c->value);
 		}
 	}
