@@ -25,7 +25,7 @@ static void get_system_name(void)
 		int rs;
 		memset(&name, 0, sizeof name);
 		EINTRLOOP(rs, uname(&name));
-		if (!rs) {
+		if (rs >= 0) {
 #ifdef OPENVMS
 			unsigned char * volatile p;
 #endif
