@@ -159,7 +159,11 @@ static void add_row_gray(unsigned *my_restrict row_buf, unsigned char *my_restri
 /* ICC misoptimizes this function when inlining it */
 ATTR_NOINLINE
 #endif
-static void add_col_color(scale_t *my_restrict col_buf, unsigned short *my_restrict ptr,
+static void add_col_color(scale_t *my_restrict col_buf,
+#if defined(__GNUC__) && __GNUC__ == 3
+	volatile
+#endif
+	unsigned short *my_restrict ptr,
 	size_t line_skip, size_t n, ulonglong weight)
 {
 	if (!weight) return;
@@ -189,7 +193,11 @@ static void add_col_color(scale_t *my_restrict col_buf, unsigned short *my_restr
 
 /* n is in pixels. pixel is 3 unsigned shorts in series */
  /* We assume unsigned short holds at least 16 bits. */
-static void add_row_color(scale_t *my_restrict row_buf, unsigned short *my_restrict ptr,
+static void add_row_color(scale_t *my_restrict row_buf,
+#if defined(__GNUC__) && __GNUC__ == 3
+	volatile
+#endif
+	unsigned short *my_restrict ptr,
 	size_t n, ulonglong weight)
 {
 	if (!weight) return;
@@ -1123,7 +1131,8 @@ static void mix_two_colors(unsigned short *my_restrict dest, unsigned char *my_r
 
 /* We assume unsigned short holds at least 16 bits. */
 void agx_and_uc_32_to_48_table(unsigned short *my_restrict dest,
-		const unsigned char *my_restrict src, size_t length, unsigned short *my_restrict table,
+		const unsigned char *my_restrict src, size_t length,
+		unsigned short *my_restrict table,
 		unsigned short rb, unsigned short gb, unsigned short bb)
 {
 	unsigned char alpha, calpha;
@@ -1141,12 +1150,12 @@ void agx_and_uc_32_to_48_table(unsigned short *my_restrict dest,
 			dest[0]=(ri*alpha+calpha*rb+127)/255;
 			dest[1]=(gi*alpha+calpha*gb+127)/255;
 			dest[2]=(bi*alpha+calpha*bb+127)/255;
-		}else{
+		} else {
 			if (alpha) {
 				dest[0]=ri;
 				dest[1]=gi;
 				dest[2]=bi;
-			}else{
+			} else {
 				dest[0]=rb;
 				dest[1]=gb;
 				dest[2]=bb;
