@@ -239,6 +239,7 @@ directfb_get_empty_bitmap (struct bitmap *bmp)
 {
   IDirectFBSurface      *surface;
   DFBSurfaceDescription  desc;
+  int skip;
 
   bmp->data = bmp->flags = NULL;
 
@@ -253,8 +254,9 @@ directfb_get_empty_bitmap (struct bitmap *bmp)
     return -1;
   }
 
-  surface->Lock (surface, (DFBSurfaceLockFlags)(DSLF_READ | DSLF_WRITE), &bmp->data, &bmp->skip);
+  surface->Lock (surface, (DFBSurfaceLockFlags)(DSLF_READ | DSLF_WRITE), &bmp->data, &skip);
 
+  bmp->skip = skip;
   bmp->flags = surface;
 
   return 0;
@@ -274,9 +276,10 @@ static void *
 directfb_prepare_strip (struct bitmap *bmp, int top, int lines)
 {
   IDirectFBSurface *surface = bmp->flags;
+  int skip;
   if (!surface) return NULL;
 
-  surface->Lock (surface, (DFBSurfaceLockFlags)(DSLF_READ | DSLF_WRITE), &bmp->data, &bmp->skip);
+  surface->Lock (surface, (DFBSurfaceLockFlags)(DSLF_READ | DSLF_WRITE), &bmp->data, &skip);
 
   return ((unsigned char *) bmp->data + top * bmp->skip);
 }
@@ -791,8 +794,8 @@ struct graphics_driver directfb_driver =
   directfb_scroll,
   directfb_set_clip_area,
   directfb_flush,
-  dummy_block,
-  dummy_unblock,
+  NULL,	 /*  block */
+  NULL,	 /*  unblock */
   NULL,	 /*  set_palette */
   NULL,	 /*  get_real_colors */
   NULL,	 /*  set_title  */
