@@ -60,7 +60,7 @@ static void grx_update_palette(void)
 	grx_driver.depth = new_depth;
 
 	grx_driver.get_color = get_color_fn(new_depth);
-	if (!grx_driver.get_color) internal("Unknown bit depth %x", new_depth);
+	if (!grx_driver.get_color) internal_error("Unknown bit depth %x", new_depth);
 
 	grx_mouse_terminate();
 	grx_set_palette();
@@ -318,7 +318,7 @@ unsupported_videomode:
 			goto unsupported_videomode;
 	}
 	grx_driver.get_color = get_color_fn(grx_driver.depth);
-	if (!grx_driver.get_color) internal("Unknown bit depth %x", grx_driver.depth);
+	if (!grx_driver.get_color) internal_error("Unknown bit depth %x", grx_driver.depth);
 	grx_driver.x = mode->width;
 	grx_driver.y = mode->height;
 	if (mode->bpp >= 8) grx_driver.flags |= GD_DONT_USE_SCROLL;
@@ -363,7 +363,7 @@ static int grx_get_empty_bitmap(struct bitmap *bmp)
 	struct grx_bmp *b;
 
 	if (!in_graphics_mode)
-		internal("grx_get_empty_bitmap: not in graphics mode");
+		internal_error("grx_get_empty_bitmap: not in graphics mode");
 
 	b = mem_alloc_mayfail(sizeof(struct grx_bmp));
 	if (!b) goto fail0;
@@ -525,7 +525,7 @@ static int grx_block(struct graphics_device *dev)
 static int grx_unblock(struct graphics_device *dev)
 {
 	if (current_virtual_device) {
-		internal("grx_unblock called without grx_block");
+		internal_error("grx_unblock called without grx_block");
 		return 0;
 	}
 	grx_set_graphics();
