@@ -2404,8 +2404,12 @@ static void do_html_textarea(unsigned char *attr, unsigned char *html, unsigned
 		mem_free(w);
 	}
 	if ((fc->maxlength = get_num(attr, cast_uchar "maxlength")) == -1) fc->maxlength = MAXINT / 4;
-	if (rows > 1) ln_break(1);
-	else put_chrs(cast_uchar " ", 1);
+	if (rows > 1) {
+		ln_break(1);
+		special_f(ff, SP_FORCE_BREAK);
+	} else {
+		put_chrs(cast_uchar " ", 1);
+	}
 	html_stack_dup();
 	get_js_events(attr);
 	format_.form = fc;
