@@ -624,7 +624,11 @@ int verify_ssl_certificate(links_ssl *ssl, unsigned char *host)
 		clear_ssl_errors(__LINE__);
 		return S_INVALID_CERTIFICATE;
 	}
+#ifdef HAVE_SSL_GET1_PEER_CERTIFICATE
+	server_cert = SSL_get1_peer_certificate(ssl->ssl);
+#else
 	server_cert = SSL_get_peer_certificate(ssl->ssl);
+#endif
 	if (!server_cert) {
 		clear_ssl_errors(__LINE__);
 		return S_INVALID_CERTIFICATE;
