@@ -268,7 +268,7 @@ static int draw_bfu_element(struct terminal * term, int x, int y, unsigned char
 			break;
 
 			default:
-			internal("draw_bfu_element: unknown BFU element type %d.\n",type);
+			internal_error("draw_bfu_element: unknown BFU element type %d.\n",type);
 		}
 		if (selected)set_char(term,x+4,y,'*',c);
 		return BFU_ELEMENT_WIDTH;  /* BFU element size in text mode */
@@ -383,7 +383,7 @@ static int draw_bfu_element(struct terminal * term, int x, int y, unsigned char
 			break;
 
 			default:
-			internal("draw_bfu_element: unknown BFU element type %d.\n",type);
+			internal_error("draw_bfu_element: unknown BFU element type %d.\n",type);
 		}
 		if (!selected)
 			drv->fill_area(dev,x+4*BFU_GRX_WIDTH,y,x+5*BFU_GRX_WIDTH,y+BFU_GRX_HEIGHT,b);
@@ -809,7 +809,7 @@ static int list_item_button(struct dialog_data *dlg, struct dialog_item_data *us
 	struct list *item = ld->current_pos;
 	struct session *ses = (struct session *)dlg->dlg->udata;
 
-	if (!ld->button_fn) internal("Links got schizophrenia! Call "BOHNICE".\n");
+	if (!ld->button_fn) internal_error("Links got schizophrenia! Call "BOHNICE".\n");
 
 	if (item == ld->list || list_empty(item->list_entry)) return 0;  /* head or empty list */
 
@@ -1049,7 +1049,7 @@ static void redraw_list_element(struct terminal *term, struct dialog_data *dlg,
 						break;
 
 					default:  /* this should never happen */
-						internal("=8-Q  lunacy level too high! Call "BOHNICE".\n");
+						internal_error("=8-Q  lunacy level too high! Call "BOHNICE".\n");
 						element = BFU_ELEMENT_EMPTY;
 
 				}
@@ -1058,7 +1058,7 @@ static void redraw_list_element(struct terminal *term, struct dialog_data *dlg,
 			mem_free(xp);
 			break;
 		default:
-			internal(
+			internal_error(
 				"Invalid list description type.\n"
 				"Somebody's probably shooting into memory.\n"
 				"_______________\n"
@@ -1154,7 +1154,7 @@ static void redraw_list_line(struct terminal *term, void *bla)
 			l = prev_in_tree(ld, ld->current_pos);
 			break;
 		default:
-			internal("redraw_list_line: invalid direction %d", direction);
+			internal_error("redraw_list_line: invalid direction %d", direction);
 			l = NULL;
 			break;
 	}
@@ -1195,7 +1195,7 @@ static void scroll_list(struct terminal *term, void *bla)
 				break;
 
 			default:
-				internal("Wrong direction %d in function scroll_list.\n",direction);
+				internal_error("Wrong direction %d in function scroll_list.\n",direction);
 		}
 
 		restrict_clip_area(term->dev, &old_area, dlg->x + DIALOG_LB, y + top, dlg->x + DIALOG_LB + w, y + bottom + G_BFU_FONT_SIZE * ld->n_items);
@@ -1716,7 +1716,7 @@ static int list_event_handler(struct dialog_data *dlg, struct links_event *ev)
 		break;
 
 		default:
-		internal("Unknown event received: %d", ev->ev);
+		break;
 
 	}
 	return EVENT_NOT_PROCESSED;
@@ -1909,7 +1909,7 @@ void reinit_list_window(struct list_description *ld)
 	ld->win_offset=ld->list;
 	ld->win_pos=0;
 
-	if (ld->open) internal("reinit_list_window: calling reinit while open");
+	if (ld->open) internal_error("reinit_list_window: calling reinit while open");
 }
 
 
