@@ -119,7 +119,7 @@ static int auth_window(struct object_request *rq, unsigned char *realm)
 {
 	unsigned char *host, *port;
 	struct dialog *d;
-	struct auth_dialog *a;
+	struct auth_dialog * volatile a;	/* avoid gcc-10 warning */
 	struct terminal *term;
 	unsigned char *urealm;
 	struct session *ses;
@@ -474,7 +474,7 @@ void release_object(struct object_request **rqq)
 void detach_object_connection(struct object_request *rq, off_t pos)
 {
 	if (rq->state == O_WAITING || rq->state == O_FAILED) {
-		internal("detach_object_connection: no data received");
+		internal_error("detach_object_connection: no data received");
 		return;
 	}
 	if (rq->refcount == 1) {
