@@ -3167,7 +3167,7 @@ int os_receive_fg_cookie(int h)
 	BOOL (WINAPI *fn_AllowSetForegroundWindow)(DWORD);
 	if (hard_read(h, (unsigned char *)&pid, sizeof pid) != sizeof pid)
 		return -1;
-	fn_AllowSetForegroundWindow = (BOOL (WINAPI *)(DWORD))GetProcAddress(GetModuleHandleA("user32.dll"), "AllowSetForegroundWindow");
+	fn_AllowSetForegroundWindow = (BOOL (WINAPI *)(DWORD))(void *)GetProcAddress(GetModuleHandleA("user32.dll"), "AllowSetForegroundWindow");
 	if (fn_AllowSetForegroundWindow)
 		fn_AllowSetForegroundWindow(pid);
 	return 0;
