@@ -125,18 +125,21 @@ int can_read(int fd)
 }
 
 
-int close_stderr(void)
+int close_std_handle(int std)
 {
 #ifndef DOS
 	int n, h, rs;
-	fflush(stderr);
+	if (std == 1)
+		fflush(stdout);
+	if (std == 2)
+		fflush(stderr);
 	n = c_open(cast_uchar "/dev/null", O_WRONLY | O_NOCTTY);
 	if (n == -1)
 		goto fail1;
-	h = c_dup(2);
+	h = c_dup(std);
 	if (h == -1)
 		goto fail2;
-	EINTRLOOP(rs, dup2(n, 2));
+	EINTRLOOP(rs, dup2(n, std));
 	if (rs == -1)
 		goto fail3;
 	EINTRLOOP(rs, close(n));
@@ -151,14 +154,17 @@ fail1:
 	return -1;
 }
 
-void restore_stderr(int h)
+void restore_std_handle(int std, int h)
 {
 #ifndef DOS
 	int rs;
-	fflush(stderr);
+	if (std == 1)
+		fflush(stdout);
+	if (std == 2)
+		fflush(stderr);
 	if (h == -1)
 		return;
-	EINTRLOOP(rs, dup2(h, 2));
+	EINTRLOOP(rs, dup2(h, std));
 	EINTRLOOP(rs, close(h));
 #endif
 }
