@@ -60,7 +60,6 @@ static_const struct s_msg_dsc msg_dsc[] = {
 
 	{S__OK,			TEXT_(T_OK)},
 	{S_INTERRUPTED,		TEXT_(T_INTERRUPTED)},
-	{S_EXCEPT,		TEXT_(T_SOCKET_EXCEPTION)},
 	{S_INTERNAL,		TEXT_(T_INTERNAL_ERROR)},
 	{S_OUT_OF_MEM,		TEXT_(T_OUT_OF_MEMORY)},
 	{S_NO_DNS,		TEXT_(T_HOST_NOT_FOUND)},
@@ -112,6 +111,7 @@ static_const struct s_msg_dsc msg_dsc[] = {
 	{S_WAIT_REDIR,		TEXT_(T_WAITING_FOR_REDIRECT_CONFIRMATION)},
 	{0,			NULL}
 };
+
 struct strerror_val {
 	list_entry_1st
 #ifdef REORDER_LIST_ENTRIES
@@ -132,14 +132,14 @@ int get_error_from_errno(int errn)
 {
 	if (errn > 0 && (errn < -S__OK || errn > -S_MAX))
 		return -errn;
-#ifdef BEOS
+#if defined(BEOS) || defined(HAIKU)
 	if (-errn > 0 && (-errn < -S__OK || -errn > -S_MAX))
 		return errn;
 #endif
 	return S_UNKNOWN_ERROR;
 }
 
-unsigned char *get_err_msg(int state)
+unsigned char *get_err_msg(int state, struct terminal *term) /* term may be NULL */
 {
 	unsigned char *e;
 	size_t sl;
@@ -152,17 +152,25 @@ unsigned char *get_err_msg(int state)
 		unk:
 		return TEXT_(T_UNKNOWN_ERROR);
 	}
-#ifdef BEOS
-	if ((e = cast_uchar strerror(state)) && *e && !strstr(cast_const_char e, "No Error")) goto have_error;
+#if defined(BEOS) || defined(HAIKU)
+	e = cast_uchar strerror_alloc(state, term);
+	if (*e && !strstr(cast_const_char e, "No Error")) goto have_error;
+	mem_free(e);
 #endif
-	if ((e = cast_uchar strerror(-state)) && *e) goto have_error;
+	e = cast_uchar strerror_alloc(-state, term);
+	if (*e) goto have_error;
+	mem_free(e);
 	goto unk;
 have_error:
-	foreach(struct strerror_val, s, ls, strerror_buf) if (!strcmp(cast_const_char s->msg, cast_const_char e)) return s->msg;
+	foreach(struct strerror_val, s, ls, strerror_buf) if (!strcmp(cast_const_char s->msg, cast_const_char e)) {
+		mem_free(e);
+		return s->msg;
+	}
 	sl = strlen(cast_const_char e);
 	if (sl > MAXINT - sizeof(struct strerror_val)) overalloc();
 	s = mem_alloc(sizeof(struct strerror_val) + sl);
 	strcpy(cast_char s->msg, cast_const_char e);
+	mem_free(e);
 	add_to_list(strerror_buf, s);
 	return s->msg;
 }
@@ -196,22 +204,31 @@ static unsigned char *get_stat_msg(struct status *stat, struct terminal *term)
 		unsigned char *m = init_str();
 		int l = 0;
 		add_to_str(&m, &l, get_text_translation(TEXT_(T_RECEIVED), term));
-		add_to_str(&m, &l, cast_uchar " ");
+		add_chr_to_str(&m, &l, ' ');
 		add_xnum_to_str(&m, &l, stat->prg->pos);
-		if (stat->prg->size >= 0)
-			add_to_str(&m, &l, cast_uchar " "), add_to_str(&m, &l, get_text_translation(TEXT_(T_OF), term)), add_to_str(&m, &l, cast_uchar " "), add_xnum_to_str(&m, &l, stat->prg->size);
+		if (stat->prg->size >= 0) {
+			add_chr_to_str(&m, &l, ' ');
+			add_to_str(&m, &l, get_text_translation(TEXT_(T_OF), term));
+			add_chr_to_str(&m, &l, ' ');
+			add_xnum_to_str(&m, &l, stat->prg->size);
+		}
 		add_to_str(&m, &l, cast_uchar ", ");
-		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME)
-			add_to_str(&m, &l, get_text_translation(TEXT_(T_AVG), term)), add_to_str(&m, &l, cast_uchar " ");
+		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME) {
+			add_to_str(&m, &l, get_text_translation(TEXT_(T_AVG), term));
+			add_chr_to_str(&m, &l, ' ');
+		}
 		add_xnum_to_str(&m, &l, stat->prg->loaded * 10 / (stat->prg->elapsed / 100));
 		add_to_str(&m, &l, cast_uchar "/s");
-		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME)
-			add_to_str(&m, &l, cast_uchar ", "), add_to_str(&m, &l, get_text_translation(TEXT_(T_CUR), term)), add_to_str(&m, &l, cast_uchar " "),
-			add_xnum_to_str(&m, &l, stat->prg->cur_loaded / (CURRENT_SPD_SEC * SPD_DISP_TIME / 1000)),
+		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME) {
+			add_to_str(&m, &l, cast_uchar ", ");
+			add_to_str(&m, &l, get_text_translation(TEXT_(T_CUR), term));
+			add_chr_to_str(&m, &l, ' ');
+			add_xnum_to_str(&m, &l, stat->prg->cur_loaded / (CURRENT_SPD_SEC * SPD_DISP_TIME / 1000));
 			add_to_str(&m, &l, cast_uchar "/s");
+		}
 		return m;
 	}
-	return stracpy(get_text_translation(get_err_msg(stat->state), term));
+	return stracpy(get_text_translation(get_err_msg(stat->state, term), term));
 }
 
 void change_screen_status(struct session *ses)
@@ -313,7 +330,10 @@ void print_screen_status(struct session *ses)
 		draw_to_window(ses->win, x_print_screen_title, ses);
 
 	m = stracpy(cast_uchar "Links");
-	if (ses->screen && ses->screen->f_data && ses->screen->f_data->title && ses->screen->f_data->title[0]) add_to_strn(&m, cast_uchar " - "), add_to_strn(&m, ses->screen->f_data->title);
+	if (ses->screen && ses->screen->f_data && ses->screen->f_data->title && ses->screen->f_data->title[0]) {
+		add_to_strn(&m, cast_uchar " - ");
+		add_to_strn(&m, ses->screen->f_data->title);
+	}
 	set_terminal_title(ses->term, m);
 	/*mem_free(m); -- set_terminal_title frees it */
 
@@ -333,7 +353,7 @@ void print_progress(struct session *ses, unsigned char *msg)
 
 void print_error_dialog(struct session *ses, struct status *stat, unsigned char *url)
 {
-	unsigned char *t = get_err_msg(stat->state);
+	unsigned char *t = get_err_msg(stat->state, ses->term);
 	unsigned char *u = display_url(ses->term, url, 1);
 	msg_box(ses->term, getml(u, NULL), TEXT_(T_ERROR), AL_CENTER, TEXT_(T_ERROR_LOADING), cast_uchar " ", u, cast_uchar ":\n\n", t, MSG_BOX_END, (void *)ses, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC /*, get_text_translation("Retry"), NULL, 0 !!! FIXME: retry */);
 }
@@ -559,34 +579,43 @@ void download_window_function(struct dialog_data *dlg)
 		m = init_str();
 		t = 1;
 		add_to_str(&m, &l, get_text_translation(TEXT_(T_RECEIVED), term));
-		add_to_str(&m, &l, cast_uchar " ");
+		add_chr_to_str(&m, &l, ' ');
 		add_xnum_to_str(&m, &l, stat->prg->pos);
-		if (stat->prg->size >= 0)
-			add_to_str(&m, &l, cast_uchar " "), add_to_str(&m, &l, get_text_translation(TEXT_(T_OF),term)), add_to_str(&m, &l, cast_uchar " "), add_xnum_to_str(&m, &l, stat->prg->size), add_to_str(&m, &l, cast_uchar " ");
+		if (stat->prg->size >= 0) {
+			add_chr_to_str(&m, &l, ' ');
+			add_to_str(&m, &l, get_text_translation(TEXT_(T_OF),term));
+			add_chr_to_str(&m, &l, ' ');
+			add_xnum_to_str(&m, &l, stat->prg->size);
+			add_chr_to_str(&m, &l, ' ');
+		}
 		add_to_str(&m, &l, cast_uchar "\n");
 		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME)
 			add_to_str(&m, &l, get_text_translation(TEXT_(T_AVERAGE_SPEED), term));
-		else add_to_str(&m, &l, get_text_translation(TEXT_(T_SPEED), term));
-		add_to_str(&m, &l, cast_uchar " ");
+		else
+			add_to_str(&m, &l, get_text_translation(TEXT_(T_SPEED), term));
+		add_chr_to_str(&m, &l, ' ');
 		add_xnum_to_str(&m, &l, (longlong)stat->prg->loaded * 10 / (stat->prg->elapsed / 100));
 		add_to_str(&m, &l, cast_uchar "/s");
-		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME)
-			add_to_str(&m, &l, cast_uchar ", "), add_to_str(&m, &l, get_text_translation(TEXT_(T_CURRENT_SPEED), term)), add_to_str(&m, &l, cast_uchar " "),
-			add_xnum_to_str(&m, &l, stat->prg->cur_loaded / (CURRENT_SPD_SEC * SPD_DISP_TIME / 1000)),
+		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME) {
+			add_to_str(&m, &l, cast_uchar ", ");
+			add_to_str(&m, &l, get_text_translation(TEXT_(T_CURRENT_SPEED), term));
+			add_chr_to_str(&m, &l, ' ');
+			add_xnum_to_str(&m, &l, stat->prg->cur_loaded / (CURRENT_SPD_SEC * SPD_DISP_TIME / 1000));
 			add_to_str(&m, &l, cast_uchar "/s");
+		}
 		add_to_str(&m, &l, cast_uchar "\n");
 		add_to_str(&m, &l, get_text_translation(TEXT_(T_ELAPSED_TIME), term));
-		add_to_str(&m, &l, cast_uchar " ");
+		add_chr_to_str(&m, &l, ' ');
 		add_time_to_str(&m, &l, stat->prg->elapsed / 1000);
 		if (stat->prg->size >= 0 && stat->prg->loaded > 0) {
 			add_to_str(&m, &l, cast_uchar ", ");
 			add_to_str(&m, &l, get_text_translation(TEXT_(T_ESTIMATED_TIME), term));
-			add_to_str(&m, &l, cast_uchar " ");
+			add_chr_to_str(&m, &l, ' ');
 			/*add_time_to_str(&m, &l, stat->prg->elapsed / 1000 * stat->prg->size / stat->prg->loaded * 1000 - stat->prg->elapsed);*/
 			/*add_time_to_str(&m, &l, (stat->prg->size - stat->prg->pos) / ((longlong)stat->prg->loaded * 10 / (stat->prg->elapsed / 100)));*/
 			add_time_to_str(&m, &l, (uttime)((stat->prg->size - stat->prg->pos) / ((double)stat->prg->loaded * 1000 / stat->prg->elapsed)));
 		}
-	} else m = stracpy(get_text_translation(get_err_msg(stat->state), term));
+	} else m = stracpy(get_text_translation(get_err_msg(stat->state, term), term));
 	show_percentage = t && test_percentage(stat);
 	u = display_url(term, down->url, 1);
 	max_text_width(term, u, &max, AL_LEFT);
@@ -822,10 +851,11 @@ time_t parse_http_date(unsigned char *date)	/* this functions is bad !!! */
 
 static void download_file_error(struct download *down, int err)
 {
-	if (get_download_ses(down)) {
-		unsigned char *emsg = stracpy(err ? cast_uchar strerror(err) : cast_uchar "Zero returned");
+	struct session *ses = get_download_ses(down);
+	if (ses) {
+		unsigned char *emsg = err ? strerror_alloc(err, ses->term) : stracpy(cast_uchar "Zero returned");
 		unsigned char *msg = stracpy(down->file);
-		msg_box(get_download_ses(down)->term, getml(msg, emsg, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_COULD_NOT_WRITE_TO_FILE), cast_uchar " ", msg, cast_uchar ": ", emsg, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
+		msg_box(ses->term, getml(msg, emsg, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_COULD_NOT_WRITE_TO_FILE), cast_uchar " ", msg, cast_uchar ": ", emsg, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 	}
 }
 
@@ -910,12 +940,8 @@ static void download_data(struct status *stat, void *down_)
 				prev_down_url = down->url;
 				down->url = u;
 				down->stat.state = S_WAIT_REDIR;
-				if (down->win) {
-					struct links_event ev = { EV_REDRAW, 0, 0, 0 };
-					ev.x = down->win->term->x;
-					ev.y = down->win->term->y;
-					down->win->handler(down->win, &ev, 0);
-				}
+				if (down->win)
+					redraw_window(down->win);
 				cache = NC_CACHE;
 				if (!strcmp(cast_const_char down->url, cast_const_char prev_down_url) || down->redirect_cnt >= MAX_CACHED_REDIRECTS) cache = NC_RELOAD;
 				allow_flags = get_allow_flags(prev_down_url);
@@ -976,7 +1002,7 @@ have_frag:
 			}
 		}
 		if (stat->state != S__OK) {
-			unsigned char *t = get_err_msg(stat->state);
+			unsigned char *t = get_err_msg(stat->state, get_download_ses(down)->term);
 			unsigned char *tt = display_url(get_download_ses(down)->term, down->url, 1);
 			msg_box(get_download_ses(down)->term, getml(tt, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_ERROR_DOWNLOADING), cast_uchar " ", tt, cast_uchar ":\n\n", t, MSG_BOX_END, (void *)get_download_ses(down), 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC /*, TEXT_(T_RETRY), NULL, 0 !!! FIXME: retry */);
 		} else {
@@ -1020,12 +1046,8 @@ have_frag:
 		abort_download(down);
 		return;
 	}
-	if (down->win) {
-		struct links_event ev = { EV_REDRAW, 0, 0, 0 };
-		ev.x = down->win->term->x;
-		ev.y = down->win->term->y;
-		down->win->handler(down->win, &ev, 0);
-	}
+	if (down->win)
+		redraw_window(down->win);
 }
 
 unsigned char *translate_download_file(unsigned char *fi)
@@ -1084,7 +1106,7 @@ int create_download_file(struct session *ses, unsigned char *cwd, unsigned char
 		}
 		if (!ses) goto x;
 		msg = stracpy(file);
-		msge = stracpy(cast_uchar strerror(errn));
+		msge = strerror_alloc(errn, ses->term);
 		msg_box(ses->term, getml(msg, msge, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_COULD_NOT_CREATE_FILE), cast_uchar " ", msg, cast_uchar ": ", msge, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		goto x;
 	}
@@ -1120,7 +1142,7 @@ static int create_or_append_download_file(struct session *ses, unsigned char *cw
 
 		EINTRLOOP(ls, lseek(*hp, 0, SEEK_END));
 		if (ls == (off_t)-1) {
-			unsigned char *emsg = stracpy(cast_uchar strerror(errno));
+			unsigned char *emsg = strerror_alloc(errno, ses->term);
 			unsigned char *msg = stracpy(*xl_file);
 			msg_box(ses->term, getml(msg, emsg, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_ERROR_CALLING_LSEEK_ON_FILE), cast_uchar " ", msg, cast_uchar ": ", emsg, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 			goto err_close;
@@ -1385,7 +1407,7 @@ unsigned long formatted_info(int type)
 				count_frames(ses->screen, &i);
 			return i;
 		default:
-			internal("formatted_info: bad request");
+			internal_error("formatted_info: bad request");
 	}
 	return 0;
 }
@@ -1452,7 +1474,7 @@ static void detach_f_data(struct f_data **ff)
 	}
 }
 
-static int shrink_format_cache(int u)
+int shrink_format_cache(int u)
 {
 	static int sc = 0;
 	int scc;
@@ -1566,7 +1588,7 @@ struct f_data *cached_format_html(struct f_data_c *fd, struct object_request *rq
 		}
 	}) {};
 	if (ses) {
-		if (report_status || !fd->f_data || fd->f_data->time_to_get >= DISPLAY_FORMATTING_STATUS)
+		if (report_status || !fd->f_data || fd->f_data->time_to_get >= DISPLAY_FORMATTING_STATUS || (rq->ce && rq->ce->length >= 1000000))
 			print_progress(ses, TEXT_(T_FORMATTING_DOCUMENT));
 	}
 	detach_f_data(&fd->f_data);
@@ -1683,12 +1705,12 @@ static void html_interpret(struct f_data_c *fd, int report_status)
 #ifdef JS
 	o.js_enable = js_enable;
 #else
-	o.js_enable=0;
+	o.js_enable = 0;
 #endif
 #ifdef G
-	o.bfu_aspect=bfu_aspect;
+	o.gamma_stamp = gamma_stamp;
 #else
-	o.bfu_aspect=0;
+	o.gamma_stamp = 0;
 #endif
 	o.plain = fd->vs->plain;
 	if (o.plain == 1 && !o.break_long_lines) {
@@ -1792,22 +1814,25 @@ void html_interpret_recursive(struct f_data_c *f)
 struct additional_file *request_additional_file(struct f_data *f, unsigned char *url_)
 {
 	size_t sl;
+	unsigned h;
 	struct additional_file *af;
-	struct list_head *laf;
 	unsigned char *u, *url;
 	url = stracpy(url_);
 	if ((u = extract_position(url))) mem_free(u);
 	if (!f->af) {
 		if (!(f->af = f->fd->af)) {
-			f->af = f->fd->af = mem_alloc(sizeof(struct additional_files));
+			f->af = f->fd->af = mem_calloc(sizeof(struct additional_files));
 			f->af->refcount = 1;
 			init_list(f->af->af);
 		}
 		f->af->refcount++;
 	}
-	foreach(struct additional_file, af, laf, f->af->af) if (!strcmp(cast_const_char af->url, cast_const_char url)) {
-		mem_free(url);
-		return af;
+	h = hash_string(url) % AF_HASH_SIZE;
+	for (af = f->af->hash[h]; af; af = af->hash_fwdlink) {
+		if (!strcmp(cast_const_char af->url, cast_const_char url)) {
+			mem_free(url);
+			return af;
+		}
 	}
 	sl = strlen(cast_const_char url);
 	if (sl > MAXINT - sizeof(struct additional_file)) overalloc();
@@ -1822,6 +1847,8 @@ struct additional_file *request_additional_file(struct f_data *f, unsigned char
 	af->need_reparse = 0;
 	af->unknown_image_size = 0;
 	add_to_list(f->af->af, af);
+	af->hash_fwdlink = f->af->hash[h];
+	f->af->hash[h] = af;
 	mem_free(url);
 	return af;
 }
@@ -1833,17 +1860,20 @@ static void copy_additional_files(struct additional_files **a)
 	struct list_head *laf;
 	if (!*a || (*a)->refcount == 1) return;
 	(*a)->refcount--;
-	afs = mem_alloc(sizeof(struct additional_files));
+	afs = mem_calloc(sizeof(struct additional_files));
 	afs->refcount = 1;
 	init_list(afs->af);
 	foreachback(struct additional_file, af, laf, (*a)->af) {
 		struct additional_file *afc;
+		unsigned h = hash_string(af->url) % AF_HASH_SIZE;
 		size_t sl = strlen(cast_const_char af->url);
 		if (sl > MAXINT - sizeof(struct additional_file)) overalloc();
 		afc = mem_alloc(sizeof(struct additional_file) + sl);
 		memcpy(afc, af, sizeof(struct additional_file) + sl);
 		if (af->rq) clone_object(af->rq, &afc->rq);
 		add_to_list(afs->af, afc);
+		afc->hash_fwdlink = afs->hash[h];
+		afs->hash[h] = afc;
 	}
 	*a = afs;
 }
@@ -2182,7 +2212,7 @@ static struct location *copy_sublocations(struct session *ses, struct location *
 		add_to_list_end(d->subframes, dl);
 		dl->parent = d;
 		z = copy_sublocations(ses, dl, sl, x);
-		if (z && y) internal("copy_sublocations: crossed references");
+		if (z && y) internal_error("copy_sublocations: crossed references");
 		if (z) y = z;
 	}
 	return y;
@@ -2193,7 +2223,7 @@ static struct location *copy_location(struct session *ses, struct location *loc)
 	struct location *l2, *l1, *nl;
 	l1 = cur_loc(ses);
 	l2 = alloc_ses_location(ses);
-	if (!(nl = copy_sublocations(ses, l2, l1, loc))) internal("copy_location: sublocation not found");
+	if (!(nl = copy_sublocations(ses, l2, l1, loc))) internal_error("copy_location: sublocation not found");
 	return nl;
 }
 
@@ -2307,13 +2337,13 @@ static void ses_go_backward(struct session *ses)
 	if (ses->search_word) mem_free(ses->search_word), ses->search_word = NULL;
 	if (ses->default_status){mem_free(ses->default_status);ses->default_status=NULL;}	/* smazeme default status, aby neopruzoval na jinych strankach */
 	reinit_f_data_c(ses->screen);
-	if (!ses->wtd_num_steps) internal("ses_go_backward: wtd_num_steps is zero");
+	if (!ses->wtd_num_steps) internal_error("ses_go_backward: wtd_num_steps is zero");
 	if (ses->wtd_num_steps > 0) {
 		n = ses->wtd_num_steps;
 		foreach(struct location, loc, lloc, ses->history) {
 			if (!n--) goto have_back_loc;
 		}
-		internal("ses_go_backward: session history disappeared");
+		internal_error("ses_go_backward: session history disappeared");
 		return;
 		have_back_loc:
 		for (n = 0; n < ses->wtd_num_steps; n++) {
@@ -2326,7 +2356,7 @@ static void ses_go_backward(struct session *ses)
 		foreach(struct location, loc, lloc, ses->forward_history) {
 			if (!++n) goto have_fwd_loc;
 		}
-		internal("ses_go_backward: session forward history disappeared");
+		internal_error("ses_go_backward: session forward history disappeared");
 		return;
 		have_fwd_loc:
 		for (n = 0; n < -ses->wtd_num_steps; n++) {
@@ -2507,7 +2537,7 @@ static int prog_sel_open(struct dialog_data *dlg, struct dialog_item_data *idata
 	struct assoc *a = (struct assoc*)idata->item->udata;
 	struct session *ses = (struct session *)dlg->dlg->udata2;
 
-	if (!a) internal("This should not happen.\n");
+	if (!a) internal_error("This should not happen.\n");
 	ses->tq_prog = stracpy(a->prog), ses->tq_prog_flag_block = a->block, ses->tq_prog_flag_direct = direct_download_possible(ses->tq, a);
 	tp_open(ses);
 	cancel_dialog(dlg,idata);
@@ -3099,8 +3129,14 @@ void win_func(struct window *win, struct links_event *ev, int fw)
 			if (ev->ev == EV_KBD || (ev->b & BM_ACT) != B_MOVE || BM_IS_WHEEL(ev->b)) move_session_to_front(ses);
 			send_event(ses, ev);
 			break;
+		case EV_EXTRA:
+			if (ev->x == EV_EXTRA_OPEN_URL) {
+				goto_url(ses, (unsigned char *)ev->b);
+				break;
+			}
+			break;
 		default:
-			error("ERROR: unknown event");
+			break;
 	}
 }
 
