@@ -253,6 +253,14 @@ void draw_to_window(struct window *win, void (*fn)(struct terminal *term, void *
 	}
 }
 
+void redraw_window(struct window *win)
+{
+	struct links_event ev = { EV_REDRAW, 0, 0, 0 };
+	ev.x = win->term->x;
+	ev.y = win->term->y;
+	win->handler(win, &ev, 0);
+}
+
 #ifdef G
 
 static void redraw_windows(void *term_)
