@@ -147,7 +147,7 @@ void g_text_draw(struct f_data_c *fd, struct g_object *t_, int x, int y)
 				return;
 			case FC_TEXT:
 			case FC_PASSWORD:
-			case FC_FILE:
+			case FC_FILE_UPLOAD:
 				if ((size_t)fs->vpos > strlen(cast_const_char fs->string)) fs->vpos = (int)strlen(cast_const_char fs->string);
 				sl = (int)strlen(cast_const_char fs->string);
 				td = textptr_diff(fs->string + fs->state, fs->string + fs->vpos, fd->f_data->opt.cp);
@@ -1334,7 +1334,7 @@ void draw_title(struct f_data_c *f)
 	}
 	w = g_text_width(bfu_style_bw, title);
 	z = 0;
-	g_print_text(dev, 0, 0, !proxies.only_proxies ? bfu_style_bw : bfu_style_wb, cast_uchar G_LEFT_ARROW, &z);
+	g_print_text(dev, 0, 0, !proxies.only_proxies ? bfu_style_bw : bfu_style_wb, cast_uchar " " G_LEFT_ARROW " ", &z);
 	f->ses->back_size = z;
 	b = (dev->size.x2 - w) - 16;
 	if (b < z) b = z;
@@ -1357,7 +1357,7 @@ static void get_searched_sub(struct g_object *p, struct g_object *c)
 	if (c->get_list) c->get_list(c, get_searched_sub);
 	if (c->draw == g_line_draw) {
 		if (srch_f_data->srch_string_size && srch_f_data->srch_string[srch_f_data->srch_string_size - 1] != ' ')
-			add_to_str(&srch_f_data->srch_string, &srch_f_data->srch_string_size, cast_uchar " ");
+			add_chr_to_str(&srch_f_data->srch_string, &srch_f_data->srch_string_size, ' ');
 	}
 }
 
@@ -1559,7 +1559,7 @@ void init_grview(void)
 	for (i = 32; i < 128; i++) {
 		unsigned char a[2];
 		a[0] = (unsigned char)i, a[1] = 0;
-		if (g_text_width(bfu_style_wb_mono, a) != w) internal("Monospaced font is not monospaced (error at char %d, width %d, wanted width %d)", i, (int)g_text_width(bfu_style_wb_mono, a), w);
+		if (g_text_width(bfu_style_wb_mono, a) != w) internal_error("Monospaced font is not monospaced (error at char %d, width %d, wanted width %d)", i, (int)g_text_width(bfu_style_wb_mono, a), w);
 	}
 #endif
 	scroll_bar_frame_color = dip_get_color_sRGB(G_SCROLL_BAR_FRAME_COLOR);
